@@ -103,50 +103,3 @@ int KonqClosedWindowItem::numTabs() const
 {
     return m_numTabs;
 }
-
-KonqClosedRemoteWindowItem::KonqClosedRemoteWindowItem(const QString &title, KConfig *config,
-        const QString &groupName, const QString &configFileName, quint64 serialNumber,
-        int numTabs, const QString &dbusService)
-    : KonqClosedWindowItem(title, config, serialNumber, numTabs),
-      m_remoteGroupName(groupName), m_remoteConfigFileName(configFileName),
-      m_dbusService(dbusService), m_remoteConfigGroup(nullptr), m_remoteConfig(nullptr)
-{
-    qCDebug(KONQUEROR_LOG);
-}
-
-KonqClosedRemoteWindowItem::~KonqClosedRemoteWindowItem()
-{
-    delete m_remoteConfigGroup;
-    delete m_remoteConfig;
-}
-
-void KonqClosedRemoteWindowItem::readConfig() const
-{
-    // only do this once
-    if (m_remoteConfig || m_remoteConfigGroup) {
-        return;
-    }
-
-    m_remoteConfig = new KConfig(m_remoteConfigFileName, KConfig::SimpleConfig);
-    m_remoteConfigGroup = new KConfigGroup(m_remoteConfig, m_remoteGroupName);
-    qCDebug(KONQUEROR_LOG);
-}
-
-const KConfigGroup &KonqClosedRemoteWindowItem::configGroup() const
-{
-    readConfig();
-    return *m_remoteConfigGroup;
-}
-
-KConfigGroup &KonqClosedRemoteWindowItem::configGroup()
-{
-    readConfig();
-    return *m_remoteConfigGroup;
-}
-
-bool KonqClosedRemoteWindowItem::equalsTo(const QString &groupName,
-        const QString &configFileName) const
-{
-    return (m_remoteGroupName == groupName &&
-            m_remoteConfigFileName == configFileName);
-}
