@@ -37,6 +37,7 @@
 #include <KLocalizedString>
 #include <KIO/JobUiDelegateFactory>
 #include <KJobTrackerInterface>
+#include <KDesktopFile>
 
 #include <QDebug>
 #include <QArgument>
@@ -396,7 +397,7 @@ bool UrlLoader::isUrlExecutable() const
         return false;
     }
 
-    return QFileInfo(m_url.path()).isExecutable();
+    return QFileInfo(m_url.path()).isExecutable() || isDesktopFileExecutable(m_url.path());
 }
 
 
@@ -517,6 +518,9 @@ void UrlLoader::downloadForEmbeddingOrOpeningDone(KonqInterfaces::DownloadJob *j
 
 void UrlLoader::checkDownloadedMimetype()
 {
+    if (m_request.forceMimeType) {
+        return;
+    }
     QMimeDatabase db;
     QMimeType type = db.mimeTypeForFile(m_url.path());
     QString typeName = type.name();
@@ -730,9 +734,34 @@ void UrlLoader::embed()
         m_mimeType = QStringLiteral("text/html");
         m_part = findPartById(QStringLiteral("webenginepart"));
     }
-    bool embedded = m_mainWindow->openView(m_mimeType, m_url, m_view, m_request, m_originalUrl);
+
+    //We need a copy of the request because we need to force embedding: KonqMainWindow::openView()
+    //uses KonqFMSettings::shouldEmbed() to determine whether to embed an URL, but if we're here
+    //it means that the user already decided to embed it and that choice should override what
+    //KonqFMSettings says. However, we don't want to call m_request.forceEmbed to avoid loosing
+    //information about available actions which are still useful in case embedding fails.
+    KonqOpenURLRequest req{m_request};
+    req.forceEmbed();
+
+    bool embedded = m_mainWindow->openView(m_mimeType, m_url, m_view, req, m_originalUrl);
     if (embedded) {
         done();
+        return;
+    }
+    //There are two reasons, that I'm aware of, which could cause openView to return false:
+    //- the view is locked and there aren't linked views
+    //- an error occurs while creating the part
+    //The first case is handled by calling KonqMainWindow::openUrl again and forcing the use of a new tab
+    //In the second case, try again to decide an action, but only among Open or Save (since embedding failed)
+
+    //TODO: shouldn't these failures be handled somewhere else, where there are more information about the failure?
+    //This is especially true for the first situation (and maybe other like it), but I think it would require a
+    //better separation of roles between main window, view and url loader
+    if (m_view->isLockedLocation() && m_view->linkedViews().isEmpty()) {
+        m_request.browserArgs.setNewTab(true);
+        m_request.browserArgs.setForcesNewWindow(true);
+        m_mainWindow->openUrl(nullptr, m_url, m_request);
+        done();
     } else {
         m_allowedActions.allow(UrlAction::Embed, false);
         m_request.serviceName = {};
@@ -868,6 +897,12 @@ bool UrlLoader::isTextExecutable(const QString &mimeType)
     return ( mimeType == QLatin1String("application/x-desktop") || mimeType == QLatin1String("application/x-shellscript"));
 }
 
+bool UrlLoader::isDesktopFileExecutable(const QString& path)
+{
+    return KDesktopFile::isDesktopFile(path) && KDesktopFile::isAuthorizedDesktopFile(path) &&
+        KDesktopFile(path).hasApplicationType();
+}
+
 QString UrlLoader::partForLocalFile(const QString& path)
 {
     QMimeDatabase db;
@@ -877,6 +912,12 @@ QString UrlLoader::partForLocalFile(const QString& path)
     return plugin.pluginId();
 }
 
+QString UrlLoader::partForMimetype(const QString& mimetype)
+{
+    KPluginMetaData plugin = preferredPart(mimetype);
+    return plugin.pluginId();
+}
+
 UrlLoader::ViewToUse UrlLoader::viewToUse() const
 {
     if (m_view && m_view->isFollowActive()) {
