@@ -56,6 +56,36 @@ public:
 
     QString defaultHttpUserAgent() const;
 
+    /** @brief Disables WebEnginePage lifecycle state management
+     *
+     * When lifecycle state management is disabled, the page will always be in the `Active` state instead of
+     * switching to the `Frozen` state when possible to reduce resource usage.
+     *
+     * By default, lifecycle state management is enabled.
+     *
+     * @warning This function is only meant to be called in tests. In tests, usually we don't want widgets of
+     * any kind to be visible. However, if a WebEnginePage isn't associated with a visible view, it'll be switched
+     * to the `Frozen` state, which (among other things) prevents it from executing javascript code. In turns, this
+     * prevents WebEnginePart from emitting the WebEnginePart::completed signal, which could cause tests to fail.
+     * Calling this function prevents the page to be `Frozen`.
+     * @note This function doesn't cause pages to automatically switch to the `Active` state, so you should call
+     * it before creating any page.
+     *
+     * @param disable whether lifecycle state management should be disabled or enabled. Note that if this is `true`
+     * lifecycle state management is _disabled_, while it is _enabled_ when @p disable is `false`
+     */
+    void disablePageLifecycleStateManagement(bool disable = true) {m_pageLifecycelManagementEnabled = !disable;}
+
+    /**
+     * @brief Whether WebEnginePage lifecycle state management is enabled or not
+     *
+     * Wen lifecycle state management is disabled, instances of WebEnginePage should never change their state to
+     * anything other then `Active`.
+     *
+     * @return whether WebEnginePage lifecycle state management is enabled or not
+     */
+    bool isPageLifecycleManagementEnabled() const {return m_pageLifecycelManagementEnabled;}
+
 private slots:
     void reparseConfiguration();
     void setHttpUserAgent(const QString &uaString);
@@ -153,6 +183,7 @@ private:
     NavigationRecorder *m_navigationRecorder;
     QString m_defaultUserAgent;
     static constexpr const char* s_userStyleSheetScriptName{"apply konqueror user stylesheet"};
+    bool m_pageLifecycelManagementEnabled = true;
 };
 
 #endif // WEBENGINEPARTCONTROLS_H
